/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.base;

import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.apache.commons.validator.GenericValidator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * La classe <code>SourceBeanContentHandler</code> implementa l'interfaccia
 * standard di un parser SAX. Questo parser ricostruisce un oggetto di tipo
 * <code>SourceBean</code> da uno stream XML.
 * @version 1.0, 11/03/2002
 * @author Luigi Bellio
 * @see SourceBeanAttribute
 * @see SourceBean
 */
public class SourceBeanContentHandler extends DefaultHandler {

    /**
     * Costruisce ed inizializza il parser.
     */
    public SourceBeanContentHandler() {
        _sourceBeans = new Stack();
        _sourceBean = null;
        _parentBean = null;
        _characters = new StringBuffer();
        _trim = true;
        _upperCase = true;
        _namespaceMappings = new ArrayList();
    } // public SourceBeanContentHandler()
    
    /**
     * Costruisce ed inizializza il parser.
     */
    public SourceBeanContentHandler(boolean trim) {
    	this();
        _trim = trim;
    } // public SourceBeanContentHandler()
    
    /**
     * Costruisce ed inizializza il parser.
     */
    public SourceBeanContentHandler(boolean trim, boolean upperCase) {
    	this(trim);
    	_upperCase = upperCase;
    } // public SourceBeanContentHandler()

    /**
     * Gestisce l'evento di inizio stream XML.
     */
    public void startDocument() throws SAXException {
    } // public void startDocument() throws SAXException

    /**
     * Gestisce l'evento di fine stream XML.
     */
    public void endDocument() throws SAXException {
    } // public void endDocument() throws SAXException

    /**
     * Gestisce l'evento di inizio elemento XML.
     */
    public void startElement(String namespaceURI, String localName, String rawName,
        Attributes attributes) throws SAXException {
            try {
                SourceBean currentBean = new SourceBean(localName, _trim, _upperCase);
                
                // Add namespace attributes
                currentBean.setNamespaceMappings(_namespaceMappings);
                // Reset namespace attributes, so they will not be included in childs
                _namespaceMappings = new ArrayList();
                
                for (int i = 0; i < attributes.getLength(); i++) {
                	String attrLocalName = attributes.getLocalName(i);
                	if (attrLocalName.indexOf('.') != -1) {
                		// The attribute contains a '.' character: it will create a hierarchical structure
                		// within the SourceBean
                		currentBean.setAttribute(attrLocalName, attributes.getValue(i));
                	} else {
                		// The attribute name may contain a namespace: store it for later use
                    	SourceBeanAttribute sbAttribute = new SourceBeanAttribute(attrLocalName, attributes.getQName(i), attributes.getValue(i));
                        currentBean.setAttribute(sbAttribute);
                	}
                } // for (int i = 0; i < attributes.getLength(); i++)
                if (_parentBean != null) {
                    _parentBean.setAttribute(currentBean);
                    
                    if (_characters.length() > 0) {
                    	addCharacters(_parentBean);
                        _characters = new StringBuffer();
                    } // if (_characters.length() > 0)
                }
                
                _sourceBeans.push(currentBean);
                _parentBean = currentBean;
            } // try
            catch (SourceBeanException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "SourceBeanContentHandler::startElement:", ex);
                throw new SAXException(ex.getMessage());
            } // catch (SourceBeanException ex) try
    } // public void startElement(String namespaceURI, String localName,
    // String rawName, Attributes attributes) throws SAXException

    /**
     * Check if the <code>SourceBean</code> already contain <code>characters</code>
     * and in this case add values to existing characters, otherwise set new ones.
     */
    private void addCharacters(SourceBean bean) {
    	if (bean == null) { return; }
    	String beanCh = bean.getCharacters();
    	if (!GenericValidator.isBlankOrNull(beanCh)) {
    		StringBuffer newCharacters = new StringBuffer(beanCh)
    			.append(_characters);
    		bean.setCharacters(newCharacters.toString());
    	} else {
    		bean.setCharacters(_characters.toString());
    	}//if (!GenericValidator.isBlankOrNull(beanCh))
    }//private void addCharacters(SourceBean bean)
    
    /**
     * Gestisce l'evento di fine elemento XML.
     */
    public void endElement(String namespaceURI, String localName, String rawName) throws SAXException {
        if (_characters.length() > 0) {
        	addCharacters(_parentBean);
            _characters = new StringBuffer();
        } // if (_characters.length() > 0)
        _sourceBean = (SourceBean)_sourceBeans.pop();
        if (_sourceBeans.empty())
            _parentBean = null;
        else
            _parentBean = (SourceBean)_sourceBeans.peek();
    } // public void endElement(String namespaceURI, String localName, String rawName) throws SAXException

    /**
     * Gestisce il testo di un elemento XML.
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        try {
            _characters.append(ch, start, length);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "SourceBeanContentHandler::characters: _characters.append(ch, start, length)", ex);
            throw new SAXException(ex.getMessage());
        } // catch (Exception ex) try
    } // public void characters(char[] ch, int start, int length) throws SAXException

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#startPrefixMapping(java.lang.String, java.lang.String)
	 */
	public void startPrefixMapping(String prefix, String uri) throws SAXException {
		_namespaceMappings.add(new NamespaceMapping(prefix, uri));
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#endPrefixMapping(java.lang.String)
	 */
	public void endPrefixMapping(String prefix) throws SAXException {
		// This code should be not necessary because the invocation order should be:
		//		startPrefixMapping
		//			startElement <- HERE the mappings are used and removed
		//			endElement
		//		endPrefixMapping
		//
		// This means that usually the mappings list in endPrefixMapping "should be"
		// empty.
		
		for (Iterator it = _namespaceMappings.iterator(); it.hasNext(); ) {
			NamespaceMapping mapping = (NamespaceMapping)it.next();
			if (mapping.getPrefix().equals(prefix)) {
				it.remove();
				break;
			}
		}
	}

	/**
     * Ritorna il <code>SourceBean</code> costruito a partire dallo stream XML.
     */
    public synchronized SourceBean getSourceBean() {
        return _sourceBean;
    } // public synchronized SourceBean getSourceBean()

    /**
     * Stack dei <code>SourceBean</code> intermedi.
     */
    private Stack _sourceBeans = null;

    /**
     * <code>SourceBean</code> costruito a partire dallo stream XML.
     */
    private SourceBean _sourceBean = null;

    /**
     * <code>SourceBean</code> padre di quello corrente.
     */
    private SourceBean _parentBean = null;
    private StringBuffer _characters = null;
    private boolean _trim = true;
    private boolean _upperCase = true;
    private List _namespaceMappings = null;
} // public class SourceBeanContentHandler extends DefaultHandler
